'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _yosay = require('yosay');

var _yosay2 = _interopRequireDefault(_yosay);

var _base = require('../../lib/base');

var _base2 = _interopRequireDefault(_base);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-disable no-console */

var Generator = _base2.default.extend({
  constructor: function constructor() {
    _base2.default.apply(this, arguments);
    this.sourceRoot(_path2.default.join(__dirname, '../../../templates/app'));
  },


  prompting: {
    askFor: function askFor() {
      var config = this.config;

      var use = config.get('use');
      if (!this.options['skip-welcome-message']) {
        // Have Yeoman greet the user.
        this.log((0, _yosay2.default)('Welcome to the marvelous ExpressDevelop generator!'));
      }
      var prompts = [{
        type: 'checkbox',
        name: 'features',
        message: 'What more would you like?',
        choices: [{
          name: 'Modernizr',
          value: 'modernizr',
          checked: use.modernizr
        }, {
          name: 'Foundation',
          value: 'foundation',
          checked: use.foundation
        }, {
          name: 'Bootstrap',
          value: 'bootstrap',
          checked: use.bootstrap
        }, {
          name: 'Backbone',
          value: 'backbone',
          checked: use.backbone
        }, {
          name: 'Marionette',
          value: 'marionette',
          checked: use.marionette
        }, {
          name: 'Sass',
          value: 'sass',
          checked: use.sass
        }, {
          name: 'Cache-crusher',
          value: 'crusher',
          checked: use.crusher
        }]
      }];
      // console.log "prompts=#{JSON.stringify(prompts)}"
      return this.prompt(prompts).then(function (answers) {
        var hasFeature = function hasFeature(feat) {
          return answers.features.indexOf(feat) !== -1;
        };

        // console.log "answers=#{JSON.stringify(answers)}"
        use.modernizr = hasFeature('modernizr');
        use.foundation = hasFeature('foundation');
        use.bootstrap = hasFeature('bootstrap');
        use.backbone = hasFeature('backbone');
        use.marionette = hasFeature('marionette');
        use.sass = hasFeature('sass');
        use.crusher = hasFeature('crusher');
        return config.set('use', use);
      });
    }
  },

  configuring: {
    saveConfig: function saveConfig() {
      this.config.set(this.settings);
      this.config.save();
    }
  },

  writing: {
    projectFiles: function projectFiles() {
      console.log('copy project files');
      this._branchCopy('root');
    },
    serverFiles: function serverFiles() {
      console.log('copy server files');
      var dirs = this.config.get('dirs');
      this._branchCopy('server', dirs.serverSrc);
    },
    clientfiles: function clientfiles() {
      console.log('copy client files');
      var dirs = this.config.get('dirs');
      this._branchCopy('client', dirs.clientSrc);
    },
    testfiles: function testfiles() {
      console.log('copy test files');
      var dirs = this.config.get('dirs');
      this._branchCopy('test', dirs.test);
    }
  },
  install: function install() {
    if (!this.options['skip-install']) {
      this.installDependencies();
    }
  }
});

exports.default = Generator;