'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _slug = require('slug');

var _slug2 = _interopRequireDefault(_slug);

var _yeomanGenerator = require('yeoman-generator');

var _yeomanGenerator2 = _interopRequireDefault(_yeomanGenerator);

var _ejs = require('ejs');

var _ejs2 = _interopRequireDefault(_ejs);

var _branchFinder = require('./branch-finder');

var _branchFinder2 = _interopRequireDefault(_branchFinder);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var BaseGenerator = _yeomanGenerator2.default.Base.extend({
  _getBranches: function _getBranches() {
    var use = this.config.get('use');
    return {
      modernizr: use.modernizr,
      foundation: use.foundation,
      bootstrap: use.bootstrap,
      backbone: use.backbone,
      marionette: use.marionette,
      sass: use.sass,
      crusher: use.crusher
    };
  },


  _transformerMakers: {
    template: function template() {
      var _this = this;

      return {
        name: 'template',
        src: /^%(.*)/,
        tgt: '$1',
        fn: function fn(s) {
          return _ejs2.default.render(s, _this.config.getAll());
        }
      };
    }
  },

  _getTransformers: function _getTransformers(names) {
    var transformers = [];
    for (var i = 0; i < names.length; i++) {
      var name = names[i];
      transformers.push(this._transformerMakers[name].apply(this));
    }
    return transformers;
  },
  _copyOp: function _copyOp(srcPath, tgtPath, transFns) {
    // console.log "op srcPath=#{srcPath} transFns=#{transFns}"
    var copyOptions = {};
    if (transFns && transFns.length) {
      copyOptions.process = function (contents) {
        var s = contents.toString();
        for (var i = 0; i < transFns.length; i++) {
          var transFn = transFns[i];
          s = transFn(s);
        }
        return s;
      };
    }

    this.fs.copy(srcPath, tgtPath, copyOptions);
  },
  _branchCopy: function _branchCopy(source, target, pattern) {
    // console.log "_branchCopy source=#{source} target=#{target} pattern=#{pattern}"
    var transNames = ['template'];
    var srcBase = this.sourceRoot();
    var tgtBase = target || '';
    pattern = pattern || '**/*';
    var _options = {
      pattern: _path2.default.join(source, pattern),
      tgtRelalative: source,
      branches: this._getBranches(),
      transformers: this._getTransformers(transNames),
      op: this._copyOp.bind(this)
    };
    var bf = new _branchFinder2.default(srcBase, tgtBase, _options);
    bf.run();
  },
  _initConfig: function _initConfig() {
    var config = this.config;

    config.defaults({
      appname: this.appname,
      appnameSlug: (0, _slug2.default)(this.appname),
      appnameCap: _lodash2.default.capitalize(this.appname),
      dirs: {},
      use: {} });
  },
  _optionConfig: function _optionConfig() {
    var config = this.config;
    var options = this.options;

    var dirs = config.get('dirs');
    config.set('dirs', _lodash2.default.defaults(dirs, {
      bower: options['bower-dir'],
      src: options['src-dir'],
      serverSrc: options['server-src-dir'],
      clientSrc: options['client-src-dir'],
      test: options['test-dir'],
      tmp: options['tmp-dir'],
      tmpDev: options['tmp-dev-dir'],
      tmpTest: options['tmp-test-dir'],
      dist: options['dist-dir']
    }));
    //TODO: set 'use'
  },
  _defaultConfig: function _defaultConfig() {
    var config = this.config;

    var dirs = undefined;
    var use = undefined;
    dirs = config.get('dirs');
    config.set('dirs', _lodash2.default.defaults(dirs, {
      bower: 'bower_components',
      src: 'src',
      test: 'test',
      tmp: '.tmp',
      dist: 'dist'
    }));
    dirs = config.get('dirs');
    config.set('dirs', _lodash2.default.defaults(dirs, {
      serverSrc: _path2.default.join(dirs.src, 'server'),
      clientSrc: _path2.default.join(dirs.src, 'client'),
      tmpDev: _path2.default.join(dirs.tmp, 'dev'),
      tmpTest: _path2.default.join(dirs.tmp, 'test')
    }));
    use = config.get('use');
    config.set('use', _lodash2.default.defaults(use, {
      backbone: true,
      marionette: true,
      foundation: true,
      bootstrap: false,
      modernizr: true,
      sass: true,
      crusher: true
    }));
  },
  constructor: function constructor() {
    _yeomanGenerator2.default.Base.apply(this, arguments);
    this.pkg = require('../../package.json');
    this._initConfig();
    this._optionConfig();
    this._defaultConfig();
    // console.log "..config=#{JSON.stringify(@config.getAll())}"
  }
});

exports.default = BaseGenerator;